
{           Copyright (c) 1985, 87 by Borland International, Inc.            }

program GrLink;
{ Die Grafik-Treiber und -Zeichenstze sind normalerweise in separaten
  Dateien gespeichert (.BGI und .CHR), sie werden zur Laufzeit eines
  Programms von der Diskette geladen.
  GRLINK demonstriert, wie sich diese Dateien direkt in ein Programm
  aufnehmen (d.h. in eine .EXE-Datei einbinden) lassen.

  Zum Start von GRLINK werden die folgenden Dateien und Programme bentigt,
  die sich smtlich im momentan gesetzten Directory befinden sollten:

  GRLINK.PAS                                         (Disk III)
  GRLINK.MAK    - MAKE-Datei fr GRLINK              (  " " )
  DRIVERS.PAS   - Unit mit den Grafik-Treibern       (  " " )
  *.BGI         - die Grafik-Treiber (Einzeldateien  (  " " )
  FONTS.PAS     - Unit mit den Grafik-Zeichenstzen  (  " " )
  *.CHR         - die Zeichenstze (Einzeldateien)   (  " " )
  GRAPH.TPU     - Unit Graph                         (  " " )

  MAKE.EXE                                           ( Disk II )
  BINOBJ.EXE    - verwandelt .CHR und .BGI-Dateien
                  in .OBJ-Dateien und ermglicht so
                  ihre Aufnahme in Programme         ( " " )

  TPC.EXE       - Kommandozeilen-Version des Compilers (Disk I)

------------------------------------
  START
------------------------------------
  1. Von der DOS-Kommandoebene aus geben Sie den Befehl

     make -fgrlink.mak

     Dadurch wird MAKE mit der Datei GRLINK.MAK aufgerufen und fhrt die
     folgenden Schritte aus:
     a) Konvertierung der Grafik-Treiber (.BGI) und -Zeichenstze (.CHR)
        in .OBJ-Dateien ber entsprechende Aufrufe von BINOBJ.EXE.
     b) Compilierung der Units DRIVERS.PAS und FONTS.PAS als .TPU-Dateien.
     c) Compilierung von GRLINK.PAS als .EXE-Datei.

  2. GRLINK.EXE enthlt danach smtliche Grafik-Treiber und -Zeichenstze
     direkt, d.h. ist auch ohne das Vorhandensein von .BGI- und .CHR-Dateien
     mit smtlichen Video-Adaptern lauffhig, die das Unit Graph untersttzt
     (CGA, EGA, EGA 64 K, EGA (mono), Hercules (mono), VGA, MCGA, IBM 3270 PC
      und AT&T 6400).

------------------------------------
  EINIGE ERLUTERUNGEN
------------------------------------

    GRLINK.PAS fhrt DRIVERS.TPU und FONTS.TPU mit einer uses-Anweisung auf:

      uses Drivers, Fonts;

    Beim Programmstart werden die Grafik-Treiber mit der Prozedur
    RegisterBGIDriver "registriert". Zur Demonstration wendet GRLINK diese
    Prozedur auf alle vorhandenen Treiber an). Analog dazu werden smtliche
    vorhandenen Zeichenstze mit der Prozedur RegisterBGIFont "registriert".
    Schlielich folgt der (sehr kurz gehaltene) "Beweis", da die Sache
    auch so funktioniert, wie sie sollte.

    Fr die Verwendung von GRLINK in eigenen Programmen sollten Sie die
    "Registrierung" der nicht verwendeten Treiber und Zeichenstze aus
    dem Programm entfernen (z.B. durch Einschlieen in Kommentarklammern).

    GRAPH.DOC (auf der Diskette III) enthlt eine detaillierte Beschreibung
    der Prozeduren RegisterBGIDriver und RegisterBGIFont.
}

uses Graph,   { das Grafikpaket }
     Drivers, { enthlt die Grafik-Treiber als Unit }
     Fonts;   { enthlt die Zeichenstze als Unit }
var
  GraphDriver, GraphMode, Error : Integer;

procedure Abort(Msg : string);
begin
  Writeln(Msg, ': ', GraphErrorMsg(GraphResult));
  Halt(1);
end;

begin
  { "Registrierung" aller Grafik-Treiber (aus DRIVERS.TPU) }
  if RegisterBGIdriver(@CGADriverProc) < 0 then     Abort('CGA');
  if RegisterBGIdriver(@EGAVGADriverProc) < 0 then  Abort('EGA/VGA');
  if RegisterBGIdriver(@HercDriverProc) < 0 then    Abort('Herc');
  if RegisterBGIdriver(@ATTDriverProc) < 0 then     Abort('AT&T');
  if RegisterBGIdriver(@PC3270DriverProc) < 0 then  Abort('PC 3270');


  { "Registrierung" aller Zeichenstze (aus FONTS.TPU) }
  if RegisterBGIfont(@GothicFontProc) < 0 then      Abort('Gothic');
  if RegisterBGIfont(@SansSerifFontProc) < 0 then   Abort('SansSerif');
  if RegisterBGIfont(@SmallFontProc) < 0 then       Abort('Small');
  if RegisterBGIfont(@TriplexFontProc) < 0 then     Abort('Triplex');


  GraphDriver := Detect;                  { automatische Erkennung }
  InitGraph(GraphDriver, GraphMode, '');  { Aktivierung des Grafikpakets }
  if GraphResult <> grOk then             { Fehler? }
  begin
    Writeln('Grafik-Initialisierungsfehler: ', GraphErrorMsg(GraphDriver));
    Halt(1);
  end;

  MoveTo(5, 5);
  OutText('Grafik-Treiber und -Zeichenstze sind');
  MoveTo(5, 20);
  SetTextStyle(GothicFont, HorizDir, 4);
  OutText('direkt ');
  SetTextStyle(SmallFont, HorizDir, 4);
  OutText('in dieser ');
  SetTextStyle(TriplexFont, HorizDir, 4);
  OutText('.EXE-Datei  ');
  SetTextStyle(SansSerifFont, VertDir, 4);
  OutText('enthalten!');
  Rectangle(0, 0, GetX +4 , GetY + TextWidth('enthalten!') + 3);
  SetTextStyle(GothicFont, HorizDir, 4);
  OutTextXY(0,GetMaxY-TextHeight('M'), 'Weiter mit <RETURN>...');
  Readln;
  CloseGraph;
end.
